@@ -3,7 +3,6 @@
 # @Time    : 2019/8/8 0008 13:11
 """
 所有中间件类型消费者的抽象基类。使实现不同中间件的消费者尽可能代码少。
-里面很多调用sys.stdout是由于print被全局打了猴子补丁变成可跳转。当识别到两个可跳转链接时候，第二个不能被识别，所以一些地方直接调用了sys.stdout。
 """
 from typing import List
 
@@ -25,12 +24,14 @@ import threading
 from threading import Lock, Thread
 import eventlet
 import gevent
+import asyncio
 from pymongo import IndexModel
 from pymongo.errors import PyMongoError
 
 # noinspection PyUnresolvedReferences
 from nb_log import LoggerLevelSetterMixin, LogManager, nb_print, LoggerMixin, LoggerMixinDefaultWithFileHandler
 # noinspection PyUnresolvedReferences
+from function_scheduling_distributed_framework.concurrent_pool.async_pool_executor import AsyncPoolExecutor
 from function_scheduling_distributed_framework.concurrent_pool.bounded_threadpoolexcutor import \
     BoundedThreadPoolExecutor
 from function_scheduling_distributed_framework.concurrent_pool.custom_evenlet_pool_executor import evenlet_timeout_deco, \
@@ -96,7 +97,6 @@ class FunctionResultStatus(LoggerMixin, LoggerLevelSetterMixin):
         self.time_start = time.time()
         self.time_cost = None
         self.success = False
-        self.current_thread = ConsumersManager.get_concurrent_info()
         self.total_thread = threading.active_count()
         self.set_log_level(20)
 
@@ -167,7 +167,7 @@ class ConsumersManager:
                 nb_print(t)
                 t.join()
         else:
-            if cls.global_concurrent_mode == 1:
+            if cls.global_concurrent_mode in [1, 4]:
                 for t in cls.schedulal_thread_to_be_join:
                     nb_print(t)
                     t.join()
@@ -183,23 +183,12 @@ class ConsumersManager:
 
     @classmethod
     def show_all_consumer_info(cls):
+        # nb_print(f'当前解释器内，所有消费者的信息是：\n  {cls.consumers_queue__info_map}')
         nb_print(f'当前解释器内，所有消费者的信息是：\n  {json.dumps(cls.consumers_queue__info_map, indent=4, ensure_ascii=False)}')
         for _, consumer_info in cls.consumers_queue__info_map.items():
             sys.stdout.write(
                 f'{time.strftime("%H:%M:%S")} "{consumer_info["where_to_instantiate"]}"  \033[0;30;44m{consumer_info["queue_name"]} 的消费者\033[0m\n')
 
-    @classmethod
-    def get_concurrent_info(cls):
-        concurrent_info = ''
-        if cls.global_concurrent_mode == 1:
-            concurrent_info = f'[{threading.current_thread()}  {threading.active_count()}]'
-        elif cls.global_concurrent_mode == 2:
-            concurrent_info = f'[{gevent.getcurrent()}  {threading.active_count()}]'
-        elif cls.global_concurrent_mode == 3:
-            # noinspection PyArgumentList
-            concurrent_info = f'[{eventlet.getcurrent()}  {threading.active_count()}]'
-        return concurrent_info
-
     @staticmethod
     def get_concurrent_name_by_concurrent_mode(concurrent_mode):
         if concurrent_mode == 1:
@@ -208,6 +197,8 @@ class ConsumersManager:
             return 'gevent'
         elif concurrent_mode == 3:
             return 'evenlet'
+        elif concurrent_mode == 4:
+            return 'async'
 
 
 class FunctionResultStatusPersistanceConfig(LoggerMixin):
@@ -231,6 +222,7 @@ class FunctionResultStatusPersistanceConfig(LoggerMixin):
                 'is_save_result': self.is_save_result, 'expire_seconds': self.expire_seconds}
 
 
+# noinspection DuplicatedCode,DuplicatedCode,DuplicatedCode,DuplicatedCode,DuplicatedCode,DuplicatedCode,DuplicatedCode,DuplicatedCode,DuplicatedCode,DuplicatedCode
 class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
     time_interval_for_check_do_not_run_time = 60
     BROKER_KIND = None
@@ -240,7 +232,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
     def publisher_of_same_queue(self):
         if not self._publisher_of_same_queue:
             self._publisher_of_same_queue = get_publisher(self._queue_name, consuming_function=self.consuming_function,
-                                                          broker_kind=self.BROKER_KIND)
+                                                          broker_kind=self.BROKER_KIND, log_level_int=self._log_level)
             if self._msg_expire_senconds:
                 self._publisher_of_same_queue.set_is_add_publish_time()
         return self._publisher_of_same_queue
@@ -269,7 +261,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
     # noinspection PyProtectedMember
     def __init__(self, queue_name, *, consuming_function: Callable = None, function_timeout=0,
-                 threads_num=50, concurrent_num=50, specify_threadpool=None, concurrent_mode=1,
+                 threads_num=50, concurrent_num=50, specify_concurrent_pool=None, specify_async_loop=None, concurrent_mode=1,
                  max_retry_times=3, log_level=10, is_print_detail_exception=True,
                  msg_schedule_time_intercal=0.0, qps: float = 0, msg_expire_senconds=0,
                  is_using_distributed_frequency_control=False,
@@ -288,7 +280,8 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         :param function_timeout : 超时秒数，函数运行超过这个时间，则自动杀死函数。为0是不限制。
         :param threads_num:线程或协程并发数量
         :param concurrent_num:并发数量，这个覆盖threads_num。以后会废弃threads_num参数，因为表达的意思不太准确，不一定是线程模式并发。
-        :param specify_threadpool:使用指定的线程池/携程池，可以多个消费者共使用一个线程池，不为None时候。threads_num失效
+        :param specify_concurrent_pool:使用指定的线程池/携程池，可以多个消费者共使用一个线程池，不为None时候。threads_num失效
+        :param specify_async_loop:指定的async的loop循环，设置并发模式为async才能起作用。
         :param concurrent_mode:并发模式，暂时支持 线程 、gevent、eventlet三种模式。  1线程  2 gevent 3 evenlet
         :param max_retry_times:
         :param log_level:
@@ -318,6 +311,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
         ConsumersManager.consumers_queue__info_map[queue_name] = current_queue__info_dict = copy.copy(self.init_params)
         current_queue__info_dict['consuming_function'] = str(consuming_function)  # consuming_function.__name__
+        current_queue__info_dict['specify_async_loop'] = str(specify_async_loop)
         current_queue__info_dict[
             'function_result_status_persistance_conf'] = function_result_status_persistance_conf.to_dict()
         current_queue__info_dict['class_name'] = self.__class__.__name__
@@ -340,8 +334,9 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         self.consuming_function = consuming_function
         self._function_timeout = function_timeout
         self._threads_num = concurrent_num if threads_num == 50 else threads_num  # concurrent参数优先，以后废弃threads_num参数。
-        self._concurrent_num =self._threads_num
-        self._specify_threadpool = specify_threadpool
+        self._concurrent_num = self._threads_num
+        self._specify_concurrent_pool = specify_concurrent_pool
+        self._specify_async_loop = specify_async_loop
         self._threadpool = None  # 单独加一个检测消息数量和心跳的线程
         self._concurrent_mode = concurrent_mode
         self._max_retry_times = max_retry_times
@@ -354,9 +349,11 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         self._is_send_consumer_hearbeat_to_redis = is_send_consumer_hearbeat_to_redis or is_using_distributed_frequency_control
         self._msg_expire_senconds = msg_expire_senconds
 
-        if self._concurrent_mode not in (1, 2, 3):
+        if self._concurrent_mode not in (1, 2, 3, 4):
             raise ValueError('设置的并发模式不正确')
         self._concurrent_mode_dispatcher = ConcurrentModeDispatcher(self)
+        if self._concurrent_mode == 4:
+            self._run = self._async_run
 
         self._logger_prefix = logger_prefix
         self._log_level = log_level
@@ -381,10 +378,11 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
         self._is_consuming_function_use_multi_params = is_consuming_function_use_multi_params
 
-        self._execute_task_times_every_minute = 0  # 每分钟执行了多少次任务。
-        self._lock_for_count_execute_task_times_every_minute = Lock()
-        self._current_time_for_execute_task_times_every_minute = time.time()
-        self._consuming_function_cost_time_total_every_minute = 0
+        self._unit_time_for_count = 10  # 每隔多少秒计数，显示单位时间内执行多少次，暂时固定为10秒。
+        self._execute_task_times_every_unit_time = 0  # 每单位时间执行了多少次任务。
+        self._lock_for_count_execute_task_times_every_unit_time = Lock()
+        self._current_time_for_execute_task_times_every_unit_time = time.time()
+        self._consuming_function_cost_time_total_every_unit_time = 0
 
         self._msg_num_in_broker = 0
         self._last_timestamp_when_has_task_in_queue = 0
@@ -461,7 +459,7 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         if self._is_send_consumer_hearbeat_to_redis:
             self._distributed_consumer_statistics = DistributedConsumerStatistics(self._queue_name, self.consumer_identification)
             self._distributed_consumer_statistics.run()
-        self.keep_circulating(20, block=False)(self.check_heartbeat_and_message_count)()
+        self.keep_circulating(10, block=False)(self.check_heartbeat_and_message_count)()  # 间隔时间最好比self._unit_time_for_count小整数倍，不然日志不准。
         self._redis_filter.delete_expire_filter_task_cycle()
         if self._schedule_tasks_on_main_thread:
             self.keep_circulating(1)(self._shedual_task)()
@@ -476,6 +474,24 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         """
         raise NotImplementedError
 
+    def __get_priority_conf(self, kw: dict, broker_task_config_key: str):
+        broker_task_config = kw['body'].get('extra', {}).get(broker_task_config_key, None)
+        if broker_task_config is None:
+            return getattr(self, f'_{broker_task_config_key}')
+        else:
+            return broker_task_config
+
+    def _get_concurrent_info(self):
+        concurrent_info = ''
+        if self._concurrent_mode == 1:
+            concurrent_info = f'[{threading.current_thread()}  {threading.active_count()}]'
+        elif self._concurrent_mode == 2:
+            concurrent_info = f'[{gevent.getcurrent()}  {threading.active_count()}]'
+        elif self._concurrent_mode == 3:
+            # noinspection PyArgumentList
+            concurrent_info = f'[{eventlet.getcurrent()}  {threading.active_count()}]'
+        return concurrent_info
+
     def _run(self, kw: dict, ):
         function_only_params = _delete_keys_and_return_new_dict(kw['body'], )
         if self.__get_priority_conf(kw, 'do_task_filtering') and self._redis_filter.check_value_exists(
@@ -490,25 +506,18 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                                                                 self.queue_name, self.consuming_function.__name__,
                                                                 kw['body']),
                                                             )
-        with self._lock_for_count_execute_task_times_every_minute:
-            self._execute_task_times_every_minute += 1
-            self._consuming_function_cost_time_total_every_minute += time.time() - t_start_run_fun
-            if time.time() - self._current_time_for_execute_task_times_every_minute > 60:
+        with self._lock_for_count_execute_task_times_every_unit_time:
+            self._execute_task_times_every_unit_time += 1
+            self._consuming_function_cost_time_total_every_unit_time += time.time() - t_start_run_fun
+            if time.time() - self._current_time_for_execute_task_times_every_unit_time > self._unit_time_for_count:
                 self.logger.info(
-                    f'一分钟内执行了 {self._execute_task_times_every_minute} 次函数 [ {self.consuming_function.__name__} ] ,'
-                    f'函数平均运行耗时 {round(self._consuming_function_cost_time_total_every_minute / self._execute_task_times_every_minute, 4)} 秒，预计'
-                    f'还需要 {time_util.seconds_to_hour_minute_second(self._msg_num_in_broker / self._execute_task_times_every_minute * 60)} 时间'
+                    f'{self._unit_time_for_count} 秒内执行了 {self._execute_task_times_every_unit_time} 次函数 [ {self.consuming_function.__name__} ] ,'
+                    f'函数平均运行耗时 {round(self._consuming_function_cost_time_total_every_unit_time / self._execute_task_times_every_unit_time, 4)} 秒，预计'
+                    f'还需要 {time_util.seconds_to_hour_minute_second(self._msg_num_in_broker / self._execute_task_times_every_unit_time * self._unit_time_for_count)} 时间'
                     f'才能执行完成 {self._msg_num_in_broker}个剩余的任务 ')
-                self._current_time_for_execute_task_times_every_minute = time.time()
-                self._consuming_function_cost_time_total_every_minute = 0
-                self._execute_task_times_every_minute = 0
-
-    def __get_priority_conf(self, kw: dict, broker_task_config_key: str):
-        broker_task_config = kw['body'].get('extra', {}).get(broker_task_config_key, None)
-        if broker_task_config is None:
-            return getattr(self, f'_{broker_task_config_key}')
-        else:
-            return broker_task_config
+                self._current_time_for_execute_task_times_every_unit_time = time.time()
+                self._consuming_function_cost_time_total_every_unit_time = 0
+                self._execute_task_times_every_unit_time = 0
 
     def _run_consuming_function_with_confirm_and_retry(self, kw: dict, current_retry_times,
                                                        function_result_status: FunctionResultStatus, ):
@@ -525,12 +534,17 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                 else:
                     function_result_status.result = function_run(
                         function_only_params)  # 消费函数使用单个参数，参数自身是一个字典，由键值对表示各个参数。
+                if asyncio.iscoroutine(function_result_status.result):
+                    self.logger.critical(f'异步的协程消费函数必须使用 async 并发模式并发,请设置 '
+                                         f'消费函数 {self.consuming_function.__name__} 的concurrent_mode 为4')
+                    # noinspection PyProtectedMember
+                    os._exit(4)
                 function_result_status.success = True
                 self._confirm_consume(kw)
                 if self.__get_priority_conf(kw, 'do_task_filtering'):
                     self._redis_filter.add_a_value(function_only_params)  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
                 self.logger.debug(f' 函数 {self.consuming_function.__name__}  '
-                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {function_only_params} 】。  {ConsumersManager.get_concurrent_info()}')
+                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {function_only_params} 】。  {self._get_concurrent_info()}')
             except Exception as e:
                 if isinstance(e, (PyMongoError,
                                   ExceptionForRequeue)):  # mongo经常维护备份时候插入不了或挂了，或者自己主动抛出一个ExceptionForRequeue类型的错误会重新入队，不受指定重试次数逇约束。
@@ -557,6 +571,83 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
                 p.execute()
         self._result_persistence_helper.save_function_result_to_mongo(function_result_status)
 
+    async def _async_run(self, kw: dict, ):
+        function_only_params = _delete_keys_and_return_new_dict(kw['body'], )
+        if self.__get_priority_conf(kw, 'do_task_filtering') and self._redis_filter.check_value_exists(
+                function_only_params):  # 对函数的参数进行检查，过滤已经执行过并且成功的任务。
+            self.logger.info(f'redis的 [{self._redis_filter_key_name}] 键 中 过滤任务 {kw["body"]}')
+            self._confirm_consume(kw)
+            return
+        t_start_run_fun = time.time()
+        await self._async_run_consuming_function_with_confirm_and_retry(kw, current_retry_times=0,
+                                                                        function_result_status=FunctionResultStatus(
+                                                                            self.queue_name, self.consuming_function.__name__,
+                                                                            kw['body']), )
+        # 异步调度不存在线程并发，不需要加锁。
+        self._execute_task_times_every_unit_time += 1
+        self._consuming_function_cost_time_total_every_unit_time += time.time() - t_start_run_fun
+        if time.time() - self._current_time_for_execute_task_times_every_unit_time > self._unit_time_for_count:
+            self.logger.info(
+                f'{self._unit_time_for_count} 秒内执行了 {self._execute_task_times_every_unit_time} 次函数 [ {self.consuming_function.__name__} ] ,'
+                f'函数平均运行耗时 {round(self._consuming_function_cost_time_total_every_unit_time / self._execute_task_times_every_unit_time, 4)} 秒，预计'
+                f'还需要 {time_util.seconds_to_hour_minute_second(self._msg_num_in_broker / self._execute_task_times_every_unit_time * self._unit_time_for_count)} 时间'
+                f'才能执行完成 {self._msg_num_in_broker}个剩余的任务 ')
+            self._current_time_for_execute_task_times_every_unit_time = time.time()
+            self._consuming_function_cost_time_total_every_unit_time = 0
+            self._execute_task_times_every_unit_time = 0
+
+    async def _async_run_consuming_function_with_confirm_and_retry(self, kw: dict, current_retry_times,
+                                                                   function_result_status: FunctionResultStatus, ):
+        function_only_params = _delete_keys_and_return_new_dict(kw['body'])
+        if current_retry_times < self.__get_priority_conf(kw, 'max_retry_times'):
+            function_result_status.run_times += 1
+            # noinspection PyBroadException
+            t_start = time.time()
+            try:
+                corotinue_obj = self.consuming_function(**function_only_params)
+                if not asyncio.iscoroutine(corotinue_obj):
+                    self.logger.critical(f'当前设置的并发模式为 async 并发模式，但消费函数不是异步协程函数，'
+                                         f'请不要把消费函数 {self.consuming_function.__name__} 的 concurrent_mode 设置为 4')
+                    # noinspection PyProtectedMember
+                    os._exit(444)
+                if self._function_timeout == 0:
+                    rs = await corotinue_obj
+                    # rs = await asyncio.wait_for(corotinue_obj, timeout=4)
+                else:
+                    rs = await asyncio.wait_for(corotinue_obj, timeout=self._function_timeout)
+                function_result_status.result = rs
+                function_result_status.success = True
+                self._confirm_consume(kw)
+                if self.__get_priority_conf(kw, 'do_task_filtering'):
+                    self._redis_filter.add_a_value(function_only_params)  # 函数执行成功后，添加函数的参数排序后的键值对字符串到set中。
+                self.logger.debug(f' 函数 {self.consuming_function.__name__}  '
+                                  f'第{current_retry_times + 1}次 运行, 正确了，函数运行时间是 {round(time.time() - t_start, 4)} 秒,入参是 【 {function_only_params} 】。  {dir(corotinue_obj.cr_running)}')
+            except Exception as e:
+                if isinstance(e, (PyMongoError,
+                                  ExceptionForRequeue)):  # mongo经常维护备份时候插入不了或挂了，或者自己主动抛出一个ExceptionForRequeue类型的错误会重新入队，不受指定重试次数逇约束。
+                    self.logger.critical(f'函数 [{self.consuming_function.__name__}] 中发生错误 {type(e)}  {e}，消息重新入队')
+                    time.sleep(1)  # 防止快速无限出错入队出队，导致cpu和中间件忙
+                    return self._requeue(kw)
+                self.logger.error(f'函数 {self.consuming_function.__name__}  第{current_retry_times + 1}次发生错误，'
+                                  f'函数运行时间是 {round(time.time() - t_start, 4)} 秒,\n  入参是 【 {function_only_params} 】   \n 原因是 {type(e)} {e} ',
+                                  exc_info=self.__get_priority_conf(kw, 'is_print_detail_exception'))
+                function_result_status.exception = f'{e.__class__.__name__}    {str(e)}'
+                return await self._async_run_consuming_function_with_confirm_and_retry(kw, current_retry_times + 1, function_result_status, )
+        else:
+            self.logger.critical(
+                f'函数 {self.consuming_function.__name__} 达到最大重试次数 {self.__get_priority_conf(kw, "max_retry_times")} 后,仍然失败， 入参是 【 {function_only_params} 】')
+            self._confirm_consume(kw)  # 错得超过指定的次数了，就确认消费了。
+        if self.__get_priority_conf(kw, 'is_using_rpc_mode'):
+            # print(function_result_status.get_status_dict(without_datetime_obj=True))
+            with RedisMixin().redis_db_frame.pipeline() as p:
+                # RedisMixin().redis_db_frame.lpush(kw['body']['extra']['task_id'], json.dumps(function_result_status.get_status_dict(without_datetime_obj=True)))
+                # RedisMixin().redis_db_frame.expire(kw['body']['extra']['task_id'], 600)
+                p.lpush(kw['body']['extra']['task_id'],
+                        json.dumps(function_result_status.get_status_dict(without_datetime_obj=True)))
+                p.expire(kw['body']['extra']['task_id'], 600)
+                p.execute()
+        self._result_persistence_helper.save_function_result_to_mongo(function_result_status)
+
     @abc.abstractmethod
     def _confirm_consume(self, kw):
         """确认消费"""
@@ -577,6 +668,8 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
         raise NotImplementedError
 
     def _submit_task(self, kw):
+        # print(kw)
+        # return
         if self._judge_is_daylight():
             self._requeue(kw)
             time.sleep(self.time_interval_for_check_do_not_run_time)
@@ -598,6 +691,8 @@ class AbstractConsumer(LoggerLevelSetterMixin, metaclass=abc.ABCMeta, ):
 
     def __frequency_control(self, qpsx, msg_schedule_time_intercalx):
         # 以下是消费函数qps控制代码。
+        if qpsx == 0:
+            return
         if qpsx <= 2:
             """ 原来的简单版 """
             time.sleep(msg_schedule_time_intercalx)
@@ -638,8 +733,10 @@ class ConcurrentModeDispatcher(LoggerMixin):
         self.consumer = consumerx
         if ConsumersManager.global_concurrent_mode is not None and self.consumer._concurrent_mode != ConsumersManager.global_concurrent_mode:
             ConsumersManager.show_all_consumer_info()
-            raise ValueError('由于猴子补丁的原因，同一解释器中不可以设置两种并发类型,请查看显示的所有消费者的信息，'
-                             '搜索 concurrent_mode 关键字，确保当前解释器内的所有消费者的并发模式只有一种')
+            # print({self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode})
+            if not {self.consumer._concurrent_mode, ConsumersManager.global_concurrent_mode}.issubset({1, 4}):
+                raise ValueError('由于猴子补丁的原因，同一解释器中不可以设置两种并发类型,请查看显示的所有消费者的信息，'
+                                 '搜索 concurrent_mode 关键字，确保当前解释器内的所有消费者的并发模式只有一种')
         self._concurrent_mode = ConsumersManager.global_concurrent_mode = self.consumer._concurrent_mode
         self.timeout_deco = None
         if self._concurrent_mode == 1:
@@ -666,8 +763,15 @@ class ConcurrentModeDispatcher(LoggerMixin):
         elif self._concurrent_mode == 3:
             pool_type = CustomEventletPoolExecutor
             check_evenlet_monkey_patch()
-        self.consumer._threadpool = self.consumer._specify_threadpool if self.consumer._specify_threadpool else pool_type(
-            self.consumer._threads_num)
+        elif self._concurrent_mode == 4:
+            pool_type = AsyncPoolExecutor
+        if self._concurrent_mode == 4:
+            self.consumer._threadpool = self.consumer._specify_concurrent_pool if self.consumer._specify_concurrent_pool else pool_type(
+                self.consumer._concurrent_num, loop=self.consumer._specify_async_loop)
+        else:
+            self.consumer._threadpool = self.consumer._specify_concurrent_pool if self.consumer._specify_concurrent_pool else pool_type(
+                self.consumer._concurrent_num)
+
         return self.consumer._threadpool
 
     def schedulal_task_with_no_block(self):
@@ -686,6 +790,10 @@ class ConcurrentModeDispatcher(LoggerMixin):
             elif self._concurrent_mode == 3:
                 g = eventlet.spawn(self.consumer.keep_circulating(1)(self.consumer._shedual_task), )
                 ConsumersManager.schedulal_thread_to_be_join.append(g)
+            elif self._concurrent_mode == 4:
+                t = Thread(target=self.consumer.keep_circulating(1)(self.consumer._shedual_task))
+                ConsumersManager.schedulal_thread_to_be_join.append(t)
+                t.start()
 
 
 def wait_for_possible_has_finish_all_tasks(queue_name: str, minutes: int, send_stop_to_broker=0,
